/***********************************************************************************************************
	Title:				MACRO "Table1"
	Description:  		A Table 1 will be created as a spreadsheet (&SPREADSHEET) and a SAS table (&OUT),
						with an unlimited number of groups (&CLASS). User must specify categorical 
						variables (&VARS_CAT), binary variables where the reported value=1 (&VARS_BIN)
						and continuous variabls (&VARS_CONT). Counts/Frequencies and Chi-Square test values
						will be output for categorical variables. Continuous variables will have Mean/S.D. 
						and Two Sample t-test/One Way ANOVA test output when normally distributed, and both
						Mean/S.D. and Median/Q1-Q3 with Wilcoxon-Mann Whitney test/Kruskal Wallis test when
						non-normally distributed, along with results from Shapiro-Wilk 
						(or Kolmogorov-Smirnov, dependent on sample size) test for normality.
	Warning:			This macro deletes all tables in the work library that begin with an underscore (_) 
						for housekeeping purposes 
	Date:				August 22, 2019
	Authors:			Daniel Brinton, PhD		brintond@musc.edu
						Marshall Chew, MSHI		chewm@musc.edu
	Acknowledgements:	This project was supported by the Health Resources and Services Administration (HRSA)
						of the U.S.Department of Health and Human Services (HHS) as part of the National 
						Telehealth Center of Excellence Award (U66 RH31458-01-00). The contents are those of 
						the authors and do not necessarily represent the official views of, nor an endorsement, 
						by HRSA, HHS or the U.S. Government.
	Presented at:		SESUG 2019, Paper 226-2019
	How to include		%include "C:\Location\MACRO - Table 1.sas";
	this macro code...
	
	Example call		%Table1(data=,out=,VARS_CAT=,VARS_BIN=,VARS_CONT=,CLASS=,SPREADSHEET="");


***********************************************************************************************************/


%macro Table1(DATA=, OUT=, VARS_CAT=, VARS_BIN=, VARS_CONT=, CLASS=, SPREADSHEET=);
	options nosource nonotes;
	ods exclude all;
		******* Categorical variables ***********************;
	%let x=1;
	%let catvar=0;
	%let _var= %scan(&VARS_CAT, &X);
	%if &_var^= %then %let catvar=1;
	%let _varbin= %scan(&VARS_BIN, &X);
	%if &_varbin^= %then %let VARS_CAT=&VARS_BIN. &VARS_CAT;
	%else %let VARS_CAT=&VARS_CAT;
	%let _var= %scan(&VARS_CAT, &X);
	%if &_var^= %then %let freqvar=1;
	%else %if &_var= %then %let freqvar=0;
	%do %while (&_var^= );
		/*	Frequency table*/
		proc freq data=&data;
			tables &_var * &CLASS / out=_temp&x totpct outpct missing noprint;
		run;
		/*	P Values*/
		proc freq data=&data noprint;
			tables &_var * &CLASS /chisq;
			output pchi out=_Atemp&x;
		run;
		proc sql;
			select P_PCHI*1 into :P_val from _Atemp&x;
		quit;
		/*	Combine */
		data _temp&x;
			set _temp&x _Atemp&x;
			length n_Percent $50 Variable $ 50 Level $ 50 CellStat $ 16 P_value 6.4 Comments1 $ 100;
			format P_value 6.4;
			Variable=vlabel(&_var);* Name/label (not value) of macro variable;
			Level=strip(&_var);* remove all leading/trailing blanks;
			P_value=put(P_PCHI, 6.4);
			if level='.' and P_value^=. then level='';
			drop &_var _PCHI_ DF_PCHI P_PCHI;
		run;
		/*	binary values */
		%if &_var=&_varbin %then %do;
			data _temp&x;
				set _temp&x;
				if level="." and P_value^=. then delete;
				if &class='' or (Level^="1" and level^=".") then delete;
				if level="1" then do;
					P_value=put(&P_val, 6.4);
					level=" ";
				end;
			run;
		%end;
		%let x = %eval(&x + 1);
		%let _var= %scan(&VARS_CAT, &X);
		%let _varbin= %scan(&VARS_BIN, &X);
		%if &_varbin= %then %let _varbin=_varbin;
	%end;
		%if &freqvar=1 %then %do;
			data _Freqs;
				set _temp:;
				length class $200;
				label N="n" Pct="Percent";
				N=COUNT;
				Pct=PCT_COL;
				Level=strip(level);
				Class=strip(&Class);
				Class_label=vlabel(&Class);
				if class="" or class="." then class="ZZZZZZ";
				if P_value^=. then Comments1="Chi-Squared test";
				drop &class COUNT Percent PCT_ROW PCT_COL;
			run;
		%end;
		%if &freqvar=1 %then %do;
			proc sql;
				create table _Freqs2 as 
				select class_label, class, variable, level, "N (%)" as CellStat length=16, strip(cat(strip(put(n, comma12.)), " (",strip(put(Pct, 4.1)), ")")) as n_Percent, P_value, Comments1 from _freqs 
				order by 1, 2, 3, 4;
			quit;
			proc sort data=_Freqs2;
				by variable level cellstat P_value Comments1;
			run;
			proc transpose data=_Freqs2 out=_Freqs3;* prefix=class;
				by variable level cellstat P_value Comments1;
				id class;
				var n_Percent;
			run;
			%if &catvar=1 %then %do;
			data _Freqs3;
				set _Freqs3;
				drop _NAME_ ZZZZZZ;
			run;
			%end;
			%if &catvar=0 %then %do;
			data _Freqs3;
				set _Freqs3;
				drop _NAME_;
			run;
			%end;
			/*	Final frequency table*/
			data _Freqs;
				set _Freqs3;
				if level="" and comments1="" then level="Missing";
			run;
		%end;
	******* Continuous variables ************************;
	%let x=1;
	%let _var= %scan(&VARS_CONT, &X);
	%if &_var^= %then %let meansvar=1; 
	%else %if &_var=  %then %let meansvar=0;
	proc sort data=&data; by &CLASS; run;
	%do %while (&_var^= );
		proc freq data=&data;
			tables &_var * &CLASS / out=_vlabel noprint;
		run;
		data _vlabel;
			set _vlabel;
			varlabel=vlabel(&_var);
		run;
		/*	determining number of groups*/
		ods output nlevels=_temp_group;
		Proc freq data=&data nlevels;
			tables &class;
		run;
		/*	determining normality */
		proc univariate data=&data noprint;
			var &_var;
			output out=_temp_normal probn=probn normaltest=stat n=n;
		run;
		/*	In the case where data is not present in a group */
		proc univariate data=&data noprint;
			class &class;
			var &_var;
			output out=_temp_normal1 probn=probn normaltest=stat1 n=n1;
		run;
		proc sql noprint;
			select (case when probn>=0.05 then 1 else 0 end) into :normal from _temp_normal;
			select round(probn, .001) into :prob_normal from _temp_normal;
			select (case when n<=2000 then "Shapiro-Wilk, W=" else "Kolmogorov-Smirnov, D=" end) into :normtest from _temp_normal;
			select round(stat, .001) into :stat from _temp_normal;
			select varlabel into: v_label from _vlabel;
			select (case when NLevels<=2 then 1 else 0 end) into :group from _temp_group;
			select (case when stat1=. and n1=0 then 2 else &group end) into :group from _temp_normal1;
			select (case when stat1=. and n1=0 then 1.1 end) into :P_val from _temp_normal1;
			select (case when stat1=. and n1=0 then "Statistical Test Not Applicable" else "" end) into :test from _temp_normal1;
		quit;
		/*	Normal continous variable with two or less comparing groups */
		%if &normal=1 %then %do;
			%if &group=1 %then %do;
				ods output ttests=_Atemp&x equality=_Btemp&x;
					proc ttest data=&data;
					class &CLASS;
					var &_var;
				run;
				data _Ctemp&x;
					merge _Atemp&x _Btemp&x;
					by variable;
				run;
				proc sql noprint;
					select probt*1 into :P_val from _Ctemp&x where probf<=0.05 and Variances="Unequal";
					select probt*1 into :P_val from _Ctemp&x where probf>0.05 and Variances="Equal";
					select "Two Sample t-test" into :test from _temp_group;
				quit;
			%end;
			/*	Normal continous variable with more than two comparing groups */
			%else %if &group=0 %then %do;
				ods output overallanova=_Atemp&x;
					proc glm data=&data;
						class &class;
						model &_var=&class;
						means &class;
					run;
					quit;
				proc sql noprint;
					select ProbF*1 into :P_val from _Atemp&x where Source="Model";
					select "One-Way ANOVA test" into :test from _temp_group;
				quit;
			%end;
			/*	Means data for continous normal variables */
			proc means data=&data noprint;
				by &CLASS;
				var &_var;
				output out=_Dtemp&x (drop=_type_ _freq_) mean=stat1 std=stat2;
			run;
			/*	putting it all together */
			data _Dtemp&x;
				set _Dtemp&x;
				length n_Percent $50 Variable $ 50 Level $ 50 CellStat $ 16 P_value 6.4 Comments1 $ 100;
				format P_value 6.4;
				Variable="&v_label";* Name/label (not value) of macro variable;
				CellStat="Mean  S.D.";
				n_Percent=cat(strip(put(stat1, 10.1)), "  ", strip(put(stat2, 10.1)));
				P_value=put(&P_val, 6.4);
				Comments1=cat(strip("&normtest"), strip(put(&stat, 10.3)), ", p=",strip(put(&prob_normal, 10.3)), " (Normal=", strip(put(&Normal, 1.0)),") ", strip("&test"));
				drop stat1 stat2;
			run;
		%end;
		/*	Non normally-distributed continous variables with any number of comparison groups*/
		%else %if &normal=0 %then %do;
			* If data are not normally-distributed, output both "Mean  S.D." and "Median [Q1 - Q3]" ;
			proc npar1way data=&data wilcoxon plots=none noprint;
				class &CLASS;
				var &_var;
				output out=_Atemp&x wilcoxon;
			run;
			/*	Gives either Wilcoxon or Kruskal-Wallis value depending on number of comparison groups*/
			%if &group=1 %then %do;
				proc sql noprint;
					select P2_WIL*1 into :P_val from _Atemp&x;
					select "Wilcoxon-Mann Whitney test" into :test from _temp_group;
				quit;
			%end;
			%else %if &group=0 %then %do;
				proc sql noprint;
					select P_KW*1 into :P_val from _Atemp&x;
					select "Kruskal-Wallis test" into :test from _temp_group;
				quit;
			%end;
			proc means data=&data noprint;
				by &CLASS;
				var &_var;
				output out=_Dtemp&x (drop=_type_ _freq_) median=stat1 q1=stat2 q3=stat3;
			run;
			/*	putting it all together*/
			data _Dtemp&x;
				set _Dtemp&x;
				length n_Percent $50 Variable $ 50 Level $ 50 CellStat $ 16 P_value 6.4 Comments1 $ 100;
				format P_value 6.4;
				Variable="&v_label";* Name/label (not value) of macro variable;
				CellStat="Median [Q1 - Q3]";
				P_value=put(&P_val, 6.4);
				n_Percent=cat(strip(put(stat1, 10.1)), '0D'x," [", strip(put(stat2, 10.1))," - ", strip(put(stat3, 10.1)), "]");
				Comments1=strip("&test");
				drop stat1 stat2 stat3;
			run;
			proc means data=&data noprint;
				by &CLASS;
				var &_var;
				output out=_Btemp&x (drop=_type_ _freq_) mean=stat1 std=stat2;
			run;
			data _Btemp&x;
				set _Btemp&x;
				length n_Percent $50 Variable $ 50 Level $ 50 CellStat $ 16 P_value 6.4 
				Comments1 $ 100;
				format P_value 6.4;
				Variable="&v_label";* Name/label (not value) of macro variable;
				CellStat="Mean  S.D.";
				n_Percent=cat(strip(put(stat1, 10.1)), "  ", strip(put(stat2, 10.1)));
				Comments1=cat(strip("&normtest"), strip(put(&stat, 10.3)), ", p=",strip(put(&prob_normal, 10.3)), " (Normal=", strip(put(&Normal, 1.0)),")");
				drop stat1 stat2;
			run;
			data _Dtemp&x; *Append the two datasets ("Mean  S.D." and "Median [Q1 - Q3]") for same statistic ;
				set _Dtemp&x _Btemp&x;
			run;
		%end;
		%let x= %eval(&x + 1);
		%let _var= %scan(&VARS_CONT, &X);
	%end;
	%if &meansvar=1 %then %do;
	data _MEANS;
		set _Dtemp:;
		Class=&Class;
		Class_label=vlabel(&Class);
		drop &class &_var;
	run;
	proc sort data=_MEANS;
		by variable level CellStat P_value Comments1;
	run;
	proc transpose data=_MEANS out=_MEANS2;
		by variable level CellStat P_value Comments1;
		id class;
		var n_Percent;
	run;
	data _MEANS;
		set _MEANS2;
		drop _NAME_;
	run;
	%end;
	******* Table Construction ************************;
	/*	Controlling for absence of continous or categorical variables*/
	/*	If we have both categorical an continuous variables*/
	%if &freqvar=1 and &meansvar=1 %then %do;
		data _temp999;
			set _Freqs _Means;
		run;
	%end;
	/*	If there are no categorical variables*/
	%else %if &freqvar^=1 and &meansvar=1 %then %do;
		data _temp999;
			set _Means;
		run;
	%end;
	/*	If there are no continuous variables */
	%else %if &freqvar=1 and &meansvar^=1 %then %do;
		data _temp999;
			set _Freqs;
		run;
	%end;
	proc sql noprint;
		select NAME into :_classes separated by " " 
		from dictionary.columns 
		where libname='WORK' and memname='_TEMP999' and upcase(NAME) not in ("VARIABLE", "LEVEL", "CELLSTAT", "COMMENTS1");
	QUIT;
	data _temp999;
		set _temp999;
		length Statistic $12;
		Statistic=strip(P_value);
		if Statistic="0" then Statistic="<0.0001";
		if Statistic="1" then Statistic="1.0";
		if Statistic="." then Statistic="";
		if Statistic="1.1" then Statistic="N/A";
		Comments=tranwrd(Comments1, "p=0.000", "p<0.001");*if p=0.000 replace with p<0.001;
		array vars(*) _CHARACTER_;* All columns with Character values (so all columns);
		do _i=1 to dim(vars);
			if upcase(vname(vars(_i))) not in ("VARIABLE", "LEVEL", "CELLSTAT", "COMMENTS" "STATISTIC") then do;
				if missing(vars(_i)) and level^='' then vars(_i)="";*if empty, replace with an emdash ("");
				if level='' and comments1="Chi-Squared test" then cellstat='';
			end;
		end;
		do while (level=".");
			Level="Missing";
			Comments="";
			Statistic="";
		end;
		drop _i P_value Comments1;
	run;
	**** Add the number of observations in each group (&CLASS) to the table ;
	proc sql;
		create table _temp as 
		select distinct(&Class) as name, put(count(*), 15.) as n, "1" as _group 
		from &data 
		group by 1;
	quit;
	data _temp;
		set _temp;
		n=strip(n);
	run;
	proc transpose data=_temp out=_temp2;
		by _group;
		id name;
		var n;
	run;
	data _temp2;
		set _temp2;
		Variable=" n";
		Level="";
		CellStat="";
		Statistic="";
		Comments="The number of observations in each group (&Class)";
		drop _group _NAME_;
	run;
	data &Out;
		set _temp999 _temp2;
	run;
	proc sort data=&out;
		by Variable level cellstat;
	run;
	data &Out;
		set &Out;
		drop cellstat; *order by cellstat, then drop;
	run;
	proc template;
		define style styles.excel_update;
		class body / backgroundcolor=_undef_ fontsize=11pt fontfamily=Helvetica fontweight=medium color=black;
		class systemtitle, usertext / fontfamily=Helvetica fontweight=bold color=black fontsize=11pt backgroundcolor=_undef_ textalign=l;
		class systemfooter / fontfamily=Helvetica fontweight=medium color=black fontsize=10pt fontstyle=italic textalign=l;
		class header / fontfamily=Helvetica fontweight=bold color=black fontsize=11pt bordertopstyle=solid borderbottomstyle=solid borderwidth=1pt;
		class footer / fontfamily=Helvetica fontweight=medium fontstyle=italic color=black fontsize=11pt bordertopstyle=solid borderwidth=1pt;
		class table / frame=hsides rules=groups borderbottomstyle=solid bordertopstyle=solid borderwidth=1pt;
		class data / fontfamily=Helvetica color=black fontsize=11pt fontweight=medium;
		class graph / height=2.5in width=4in;
		class graphdata1 / color=#C8DADA;
		class graphdata2 / color=#5A6351;
		class graphdata3 / color=#69B0A5;
		class graphdata4 / color=#F2F3F2;
		end;
	run;
	ods exclude none;
	%if (&SPREADSHEET^= ) %then %do; * if a location for a spreadsheet is specified, output it ;
		ods listing close;
		options leftmargin=0.5in
				rightmargin=0.5in
				topmargin=0.75in
				bottommargin=0.25in;
		ods excel file=&Spreadsheet options(sheet_interval="none" 
			embedded_titles="yes" sheet_name="Table 1" start_at="1,1" flow="tables"
			embedded_footnotes="yes" Orientation="Landscape" frozen_headers="3" Gridlines="off") 
			TITLE='Table 1 from Macro'
			COMMENTS='Macro from SESUG paper 226-2019. This project was supported by the Health Resources and Services Administration (HRSA) of the Department of Health and Human Services (HHS) as part of the National Telehealth Center of Excellence Award (U66 RH31458-01-00).'
			AUTHOR='Daniel Brinton, PhD; Marshall Chew, MSHI'
			style=styles.excel_update;
			proc report data=&Out style(column header report lines hdr calldef)=[tagattr="wrap:no" just=c vjust=t];
				title "Table 1. Demographics and characteristics (Source: &data)";
				define Variable / group 'Characteristic' style=[just=l tagattr="wrap:no"];
				define Level / display '' style=[just=l tagattr="wrap:no"];
				define Statistic / display 'p-value' style=[just=c tagattr="wrap:no"];
				define Comments / display style=[just=l tagattr="wrap:no" width=50%];
				Footnote "All values expressed as n(%), mean s.d., or median[Q1 - Q3]";
			run;
		ods excel close;
		ods listing;
	%end;
	* Delete all tables created by this macro ;
	proc sql noprint;
		select distinct MEMNAME into :tables separated by " " from DICTIONARY.COLUMNS 
		where LIBNAME='WORK' and substr(MEMNAME, 1, 1)='_';
	quit;
	%let X=1;
	%let _table= %scan(&tables, &X);
	%do %while (&_table^= );
		proc delete data=&_table; run;
		%let X= %eval(&X + 1);
		%let _table= %scan(&tables, &X);
	%end;
	title;title2;
	options source notes;
	ods output close;
%mend Table1;
